@@ -9,15 +9,17 @@ from typing import Any, Callable, Dict, Iterable, List, Optional, Pattern, Set,
 
 import attr
 import doit
-import toml
+import tomli
+from attrs_strict import type_validator
 from beartype import beartype
 from doit.action import BaseAction
 from doit.task import Task
 from loguru import logger
 
+from ..code_tag_collector import CODE_TAG_RE, COMMON_CODE_TAGS
 from ..file_helpers import _MKDOCS_CONFIG_NAME, _read_yaml_file, get_doc_dir
+from ..file_search import find_project_files, find_project_files_by_suffix
 from ..log_helpers import log_fun
-from .file_search import find_project_files, find_project_files_by_suffix
 
 _DOIT_TASK_IMPORT_ERROR = 'User must install the optional calcipy extra "dev" to utilize "doit_tasks"'
 """Standard error message when an optional import is not available. Raise with RuntimeError."""
@@ -59,13 +61,13 @@ def _member_filter(member: Any, instance_type: Any) -> bool:
         bool: True if the member matches the applied filter
 
     """
-    return (instance_type is None or isinstance(member, instance_type))
+    return instance_type is None or isinstance(member, instance_type)
 
 
 @attr.s(auto_attribs=True, kw_only=True)
 class _PathAttrBase:  # noqa: H601
 
-    path_project: Path
+    path_project: Path = attr.ib(validator=type_validator())
     """Path to the package directory."""
 
     def __attrs_post_init__(self) -> None:
@@ -131,22 +133,22 @@ class _PathAttrBase:  # noqa: H601
 class PackageMeta(_PathAttrBase):  # noqa: H601
     """Package Meta-Information."""
 
-    path_toml: Path = Path('pyproject.toml')
+    path_toml: Path = attr.ib(validator=type_validator(), default=Path('pyproject.toml'))
     """Relative path to the poetry toml file."""
 
-    ignore_patterns: List[str] = []
+    ignore_patterns: List[str] = attr.ib(validator=type_validator(), factory=list)
     """List of glob patterns to ignore from all analysis."""
 
-    paths: List[Path] = attr.ib(init=False)
+    paths: List[Path] = attr.ib(validator=type_validator(), init=False)
     """Paths to all tracked files that were not ignored with specified patterns `find_project_files`."""
 
-    paths_by_suffix: Dict[str, List[Path]] = attr.ib(init=False)
+    paths_by_suffix: Dict[str, List[Path]] = attr.ib(validator=type_validator(), init=False)
     """Paths to all tracked files that were not ignored with specified patterns `find_project_files_by_suffix`."""
 
-    pkg_name: str = attr.ib(init=False)
+    pkg_name: str = attr.ib(validator=type_validator(), init=False)
     """Package string name."""
 
-    pkg_version: str = attr.ib(init=False)
+    pkg_version: str = attr.ib(validator=type_validator(), init=False)
     """Package version."""
 
     def __attrs_post_init__(self) -> None:
@@ -160,14 +162,13 @@ class PackageMeta(_PathAttrBase):  # noqa: H601
         super().__attrs_post_init__()
 
         # Note: toml is an optional dependency required only when using the `doit_tasks` in development
-        if toml is None:  # pragma: no cover
+        if tomli is None:  # pragma: no cover
             raise RuntimeError(_DOIT_TASK_IMPORT_ERROR)
 
-        try:
-            poetry_config = toml.load(self.path_toml)['tool']['poetry']
-        except FileNotFoundError:  # pragma: no cover
-            raise FileNotFoundError(f'Check that "{self.path_project}" is correct. Could not find: {self.path_toml}')
+        if not self.path_toml.is_file():
+            raise RuntimeError(f'Check "{self.path_project}". Could not find: "{self.path_toml}"')
 
+        poetry_config = tomli.loads(self.path_toml.read_text())['tool']['poetry']
         self.pkg_name = poetry_config['name']
         self.pkg_version = poetry_config['version']
 
@@ -192,34 +193,38 @@ class PackageMeta(_PathAttrBase):  # noqa: H601
         }  # type: ignore[attr-defined]
 
 
+_DEF_IGNORE_LIST = [
+    'AAA01',  # AAA01 / act block in pytest
+    'C901',  # C901 / complexity from "max-complexity = 10"
+    'D417',  # D417 / missing arg descriptors
+    'DAR101', 'DAR201', 'DAR401',  # https://pypi.org/project/darglint/ (Scroll to error codes)
+    'DUO106',  # DUO106 / insecure use of os
+    'E800',  # E800 / Commented out code
+    'G001',  # G001 / logging format for un-indexed parameters
+    'H601',  # H601 / class with low cohesion
+    'P101', 'P103',  # P101,P103 / format string
+    'PD013',
+    'S101',  # S101 / assert
+    'S605', 'S607',  # S605,S607 / os.popen(...)
+    'T100', 'T101', 'T103',  # T100,T101,T103 / fixme and todo comments
+]
+"""Default list of excluded flake8 rules for the pre-commit check (additional to .flake8)."""
+
+
 @attr.s(auto_attribs=True, kw_only=True)
 class LintConfig(_PathAttrBase):  # noqa: H601
     """Lint Config."""
 
-    path_flake8: Union[Path, str] = Path('.flake8')
+    path_flake8: Union[Path, str] = attr.ib(validator=type_validator(), default=Path('.flake8'))
     """Relative path to the flake8 configuration file. Default is ".flake8" created by calcipy_template."""
 
-    path_isort: Union[Path, str] = Path('pyproject.toml')
+    path_isort: Union[Path, str] = attr.ib(validator=type_validator(), default=Path('pyproject.toml'))
     """Relative path to the isort configuration file. Default is "pyproject.toml" created by calcipy_template."""
 
-    ignore_errors: List[str] = [
-        'AAA01',  # AAA01 / act block in pytest
-        'C901',  # C901 / complexity from "max-complexity = 10"
-        'D417',  # D417 / missing arg descriptors
-        'DAR101', 'DAR201', 'DAR401',  # https://pypi.org/project/darglint/ (Scroll to error codes)
-        'DUO106',  # DUO106 / insecure use of os
-        'E800',  # E800 / Commented out code
-        'G001',  # G001 / logging format for un-indexed parameters
-        'H601',  # H601 / class with low cohesion
-        'P101', 'P103',  # P101,P103 / format string
-        'PD013',
-        'S101',  # S101 / assert
-        'S605', 'S607',  # S605,S607 / os.popen(...)
-        'T100', 'T101', 'T103',  # T100,T101,T103 / fixme and todo comments
-    ]
+    ignore_errors: List[str] = attr.ib(validator=type_validator(), factory=lambda: _DEF_IGNORE_LIST)
     """List of additional excluded flake8 rules for the pre-commit check."""
 
-    paths_py: List[Path] = attr.ib(init=False)
+    paths_py: List[Path] = attr.ib(validator=type_validator(), init=False)
     """Paths to the Python files used when linting. Created with `find_project_files_by_suffix`."""
 
     def __attrs_post_init__(self) -> None:
@@ -231,37 +236,40 @@ class LintConfig(_PathAttrBase):  # noqa: H601
 
 
 @attr.s(auto_attribs=True, kw_only=True)
-class TestingConfig(_PathAttrBase):  # noqa: H601
+class TestingConfig(_PathAttrBase):  # noqa: H601  # pylint: disable=too-many-instance-attributes
     """Test Config."""
 
-    pythons: List[str] = ['3.8', '3.9']
+    pythons: List[str] = attr.ib(validator=type_validator(), factory=lambda: ['3.8', '3.9'])
     """Python versions to test against. Default is `['3.8', '3.9']`."""
 
-    path_out: Union[Path, str] = Path('releases/tests')
+    path_out: Union[Path, str] = attr.ib(validator=type_validator(), default=Path('releases/tests'))
     """Relative path to the report output directory. Default is `releases/tests`."""
 
-    path_tests: Union[Path, str] = Path('tests')
+    path_tests: Union[Path, str] = attr.ib(validator=type_validator(), default=Path('tests'))
     """Relative path to the tests directory. Default is `tests`."""
 
-    args_pytest: str = '--exitfirst --showlocals --failed-first --new-first --verbose --doctest-modules'
+    args_pytest: str = attr.ib(
+        validator=type_validator(),
+        default='--exitfirst --showlocals --failed-first --new-first --verbose --doctest-modules',
+    )
     """Default arguments to Pytest. In short form, the defaults are `-x -l --ff --nf -vv`."""
 
-    args_diff: str = '--fail-under=65 --compare-branch=origin/main'
+    args_diff: str = attr.ib(validator=type_validator(), default='--fail-under=65 --compare-branch=origin/main')
     """Default arguments to diff-cover."""
 
-    path_test_report: Path = attr.ib(init=False)
+    path_test_report: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the self-contained test HTML report."""
 
-    path_diff_test_report: Path = attr.ib(init=False)
+    path_diff_test_report: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the self-contained diff-test HTML report."""
 
-    path_diff_lint_report: Path = attr.ib(init=False)
+    path_diff_lint_report: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the self-contained diff-lint HTML report."""
 
-    path_coverage_index: Path = attr.ib(init=False)
+    path_coverage_index: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the coverage HTML index file within the report directory."""
 
-    path_mypy_index: Path = attr.ib(init=False)
+    path_mypy_index: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the mypy HTML index file within the report directory."""
 
     def __attrs_post_init__(self) -> None:
@@ -282,21 +290,19 @@ class TestingConfig(_PathAttrBase):  # noqa: H601
 class CodeTagConfig(_PathAttrBase):  # noqa: H601
     """Code Tag Config."""
 
-    doc_sub_dir: Path = Path('docs/docs')
+    doc_sub_dir: Path = attr.ib(validator=type_validator(), default=Path('docs/docs'))
     """Relative path to the source documentation directory."""
 
-    code_tag_summary_filename: str = 'CODE_TAG_SUMMARY.md'
+    code_tag_summary_filename: str = attr.ib(validator=type_validator(), default='CODE_TAG_SUMMARY.md')
     """Name of the code tag summary file."""
 
-    tags: List[str] = [
-        'FIXME', 'TODO', 'PLANNED', 'HACK', 'REVIEW', 'TBD', 'DEBUG', 'FYI', 'NOTE',  # noqa: T100,T101,T103
-    ]
+    tags: List[str] = attr.ib(validator=type_validator(), factory=lambda: COMMON_CODE_TAGS)
     """List of ordered tag names to match."""
 
-    re_raw: str = r'((\s|\()(?P<tag>{tag})(:[^\r\n]))(?P<text>.+)'
+    re_raw: str = attr.ib(validator=type_validator(), default=CODE_TAG_RE)
     """string regular expression that contains `{tag}`."""
 
-    path_code_tag_summary: Path = attr.ib(init=False)
+    path_code_tag_summary: Path = attr.ib(validator=type_validator(), init=False)
     """Path to the code tag summary file. Uses `code_tag_summary_filename`."""
 
     def __attrs_post_init__(self) -> None:
@@ -319,16 +325,19 @@ class CodeTagConfig(_PathAttrBase):  # noqa: H601
 class DocConfig(_PathAttrBase):  # noqa: H601
     """Documentation Config."""
 
-    doc_sub_dir: Path = Path('docs/docs')
+    doc_sub_dir: Path = attr.ib(validator=type_validator(), default=Path('docs/docs'))
     """Relative path to the source documentation directory."""
 
-    handler_lookup: Optional[Dict[str, Callable[[str, Path], str]]] = None
+    auto_doc_path: Optional[Path] = attr.ib(validator=type_validator(), default=None)
+    """Auto-calculated based on `self.doc_sub_dir`."""
+
+    handler_lookup: Optional[Dict[str, Callable[[str, Path], str]]] = attr.ib(validator=type_validator(), default=None)
     """Lookup dictionary for autoformatted sections of the project's markdown files."""
 
-    path_out: Path = attr.ib(init=False)
+    path_out: Path = attr.ib(validator=type_validator(), init=False)
     """The documentation output directory. Specified in `mkdocs.yml`."""
 
-    paths_md: List[Path] = attr.ib(init=False)
+    paths_md: List[Path] = attr.ib(validator=type_validator(), init=False)
     """Paths to Markdown files used when documenting. Created with `find_project_files_by_suffix`."""
 
     def __attrs_post_init__(self) -> None:
@@ -339,30 +348,34 @@ class DocConfig(_PathAttrBase):  # noqa: H601
         self.path_out = _make_full_path(self.path_out, self.path_project)
         self.path_out.mkdir(exist_ok=True, parents=True)
         self.paths_md = DG.meta.paths_by_suffix.get('md', [])
+        self.auto_doc_path = self.doc_sub_dir.parent / 'modules'
 
 
 @attr.s(auto_attribs=True, kw_only=True)
-class DoitGlobals:  # noqa: H601
+class DoitGlobals:  # noqa: H601  # pylint: disable=too-many-instance-attributes
     """Global Variables for doit."""
 
-    calcipy_dir: Path = attr.ib(init=False, default=Path(__file__).resolve().parents[1])
+    calcipy_dir: Path = attr.ib(validator=type_validator(), init=False, default=Path(__file__).resolve().parents[1])
     """The calcipy directory (likely within `.venv`)."""
 
-    meta: PackageMeta = attr.ib(init=False)
+    meta: PackageMeta = attr.ib(validator=type_validator(), init=False)
     """Package Meta-Information."""
 
-    lint: LintConfig = attr.ib(init=False)
+    lint: LintConfig = attr.ib(validator=type_validator(), init=False)
     """Lint Config."""
 
-    test: TestingConfig = attr.ib(init=False)
+    test: TestingConfig = attr.ib(validator=type_validator(), init=False)
     """Test Config."""
 
-    ct: CodeTagConfig = attr.ib(init=False)
+    tags: CodeTagConfig = attr.ib(validator=type_validator(), init=False)
     """Documentation Config."""
 
-    doc: DocConfig = attr.ib(init=False)
+    doc: DocConfig = attr.ib(validator=type_validator(), init=False)
     """Documentation Config."""
 
+    _is_set: bool = attr.ib(validator=type_validator(), default=False)
+    """Internal flag to check if already set."""
+
     @log_fun
     def set_paths(
         self, *, path_project: Optional[Path] = None,
@@ -372,18 +385,27 @@ class DoitGlobals:  # noqa: H601
         Args:
             path_project: optional project base directory Path. Defaults to the current working directory
 
+        Raises:
+            RuntimeError: if `set_paths` is called twice. This can cause unexpected behavior, so if
+                absolutely necessary, set `DG._is_set = False`, then try `DG.set_paths()`
+
         """
+        if self._is_set:
+            raise RuntimeError('DoitGlobals has already been configured and cannot be reconfigured')
+
         logger.info(f'Setting DG path: {path_project}', path_project=path_project, cwd=Path.cwd())
         path_project = path_project or Path.cwd()
 
         # Read the optional toml configuration
         # > Note: could allow LintConfig/.../DocConfig kwargs to be set in toml, but may be difficult to maintain
-        path_toml = path_project / 'pyproject.toml'
-        calcipy_config = toml.load(path_toml).get('tool', {}).get('calcipy', {})
+        data = (path_project / 'pyproject.toml').read_text()
+        calcipy_config = tomli.loads(data).get('tool', {}).get('calcipy', {})
 
         self._set_meta(path_project, calcipy_config)
         self._set_submodules(calcipy_config)
 
+        self._is_set = True
+
     def _set_meta(self, path_project: Path, calcipy_config: Dict[str, Any]) -> None:
         """Initialize the meta submodules.
 
@@ -418,10 +440,10 @@ class DoitGlobals:  # noqa: H601
 
         # Configure submodules
         meta_kwargs = {'path_project': self.meta.path_project}
-        lint_k, test_k, code_k, doc_k = [calcipy_config.get(key, {}) for key in section_keys]
+        lint_k, test_k, code_k, doc_k = (calcipy_config.get(key, {}) for key in section_keys)
         self.lint = LintConfig(**meta_kwargs, **lint_k)  # type: ignore[arg-type]
         self.test = TestingConfig(**meta_kwargs, **test_k)  # type: ignore[arg-type]
-        self.ct = CodeTagConfig(**meta_kwargs, doc_sub_dir=doc_sub_dir, **code_k)  # type: ignore[arg-type]
+        self.tags = CodeTagConfig(**meta_kwargs, doc_sub_dir=doc_sub_dir, **code_k)  # type: ignore[arg-type]
         self.doc = DocConfig(**meta_kwargs, doc_sub_dir=doc_sub_dir, **doc_k)  # type: ignore[arg-type]
 
 
