@@ -5,9 +5,11 @@ from pathlib import Path
 from typing import Dict, List, Optional
 
 import attr
+import numpy as np
 import pendulum
 import requests
-import toml
+import tomli
+from attrs_strict import type_validator
 from beartype import beartype
 from doit.tools import Interactive
 from loguru import logger
@@ -34,7 +36,7 @@ def task_check_license() -> DoitTask:
 
 @beartype
 def task_lock() -> DoitTask:
-    """Lock dependencies.
+    """Ensure poetry.lock and requirements.txt are up-to-date.
 
     Returns:
         DoitTask: doit task
@@ -42,11 +44,12 @@ def task_lock() -> DoitTask:
     """
     path_req = DG.meta.path_project / 'requirements.txt'
     # Ensure that extras are exported as well
-    toml_data = toml.loads(DG.meta.path_toml.read_text())
+    toml_data = tomli.loads(DG.meta.path_toml.read_text())
+    # FYI: poetry 'groups' appear to be properly exported with "--dev"
     extras = [*toml_data['tool']['poetry'].get('extras', {}).keys()]
     extras_arg = ' -E '.join([''] + extras) if extras else ''
     task = debug_task([
-        'poetry lock',
+        'poetry lock --no-update',
         f'poetry export -f {path_req.name} -o {path_req.name}{extras_arg} --dev',
     ])
     task['file_dep'].append(DG.meta.path_toml)
@@ -131,27 +134,27 @@ def _auto_convert(_cls, fields):  # type: ignore # noqa: ANN001, ANN202, CCR001
 class _HostedPythonPackage():  # noqa: H601
     """Representative information for a python package hosted on some domain."""
 
-    domain: str = 'https://pypi.org/pypi/{name}/{version}/json'
-    name: str
-    version: str
-    datetime: Optional[DateTime] = None
-    latest_version: str = ''
-    latest_datetime: Optional[DateTime] = None
+    domain: str = attr.ib(validator=type_validator(), default='https://pypi.org/pypi/{name}/{version}/json')
+    name: str = attr.ib(validator=type_validator())
+    version: str = attr.ib(validator=type_validator())
+    datetime: Optional[DateTime] = attr.ib(validator=type_validator(), default=None)
+    latest_version: str = attr.ib(validator=type_validator(), default='')
+    latest_datetime: Optional[DateTime] = attr.ib(validator=type_validator(), default=None)
 
 
 _PATH_PACK_LOCK = DG.meta.path_project / '.calcipy_packaging.lock'
 """Path to the packaging lock file."""
 
-# PLANNED: Check that this prevent excess requests to PyPi and refactor
+# HACK: Check that this works then refactor (shouldn't be global)
 #   https://pypi.org/project/pyrate-limiter/
 #   https://learn.vonage.com/blog/2020/10/22/respect-api-rate-limits-with-a-backoff-dr/
-rate = RequestRate(3, 5 * Duration.SECOND)
-limiter = Limiter(rate)
-item = 'pypi'
+_RATE = RequestRate(3, 5 * Duration.SECOND)
+_LIMITER = Limiter(_RATE)
+_ITEM = 'pypi'
 
 
 @beartype
-@limiter.ratelimit(item, delay=True, max_delay=10)
+@_LIMITER.ratelimit(_ITEM, delay=True, max_delay=10)
 def _get_release_date(package: _HostedPythonPackage) -> _HostedPythonPackage:
     """Retrieve release date metadata for the specified package.
 
@@ -164,7 +167,8 @@ def _get_release_date(package: _HostedPythonPackage) -> _HostedPythonPackage:
     """
     # Retrieve the JSON summary for the specified package
     json_url = package.domain.format(name=package.name, version=package.version)
-    res = requests.get(json_url)
+    res = requests.get(json_url, timeout=30)
+    res.raise_for_status()
     releases = res.json()['releases']
     package.datetime = pendulum.parse(releases[package.version][0]['upload_time_iso_8601'])
 
@@ -220,13 +224,16 @@ def _collect_release_dates(
 
     updated_packages = []
     for package in packages:
-        cached_package = old_cache.get(package.name)
-        cached_version = '' if cached_package is None else cached_package.version
-        if package.version != cached_version:
-            package = _get_release_date(package)
-        else:
-            package = cached_package
-        updated_packages.append(package)
+        try:
+            cached_package = old_cache.get(package.name)
+            cached_version = '' if cached_package is None else cached_package.version
+            if package.version != cached_version:
+                package = _get_release_date(package)
+            else:
+                package = cached_package
+            updated_packages.append(package)
+        except requests.exceptions.HTTPError as err:
+            logger.warning(f'Could not resolve {package} with error {err}')
     return updated_packages
 
 
@@ -239,10 +246,8 @@ def _write_cache(updated_packages: List[_HostedPythonPackage], path_pack_lock: P
         path_pack_lock: Path to the lock file. Default is `_PATH_PACK_LOCK`
 
     """
-    def serialize(inst, field, value):  # noqa: ANN001, ANN201
-        if isinstance(value, DateTime):
-            return value.to_iso8601_string()
-        return value
+    def serialize(_inst, _field, value):  # noqa: ANN001, ANN201
+        return value.to_iso8601_string() if isinstance(value, DateTime) else value
 
     new_cache = {pack.name: attr.asdict(pack, value_serializer=serialize) for pack in updated_packages}
     pretty_json = json.dumps(new_cache, indent=4, separators=(',', ': '), sort_keys=True)
@@ -266,7 +271,7 @@ def _read_packages(path_lock: Path) -> List[_HostedPythonPackage]:
     if path_lock.name != 'poetry.lock':
         raise NotImplementedError(f'{path_lock.name} is not a currently supported lock type. Try "poetry.lock" instead')
 
-    lock = toml.loads(path_lock.read_text())
+    lock = tomli.loads(path_lock.read_text())
     # TBD: Handle non-pypi domains and format the URL accordingly (i.e. TestPyPi, etc.)
     # > domain=dependency['source']['url'] + '{name}/{version}/json'
     return [
@@ -296,6 +301,9 @@ def _check_for_stale_packages(packages: List[_HostedPythonPackage], *, stale_mon
     if stale_packages:
         stale_list = '\n'.join(map(format_package, sorted(stale_packages, key=lambda x: x.datetime)))
         logger.warning(f'Found stale packages that may be a dependency risk:\n\n{stale_list}\n\n')
+    else:
+        max_months = np.amax([now.diff(pack.datetime).in_months() for pack in packages])
+        logger.warning(f'The oldest package was released {max_months} months ago (stale is >{stale_months} months)\n')
 
 
 @beartype
@@ -330,7 +338,7 @@ def task_check_for_stale_packages() -> DoitTask:
     path_pack_lock = _PATH_PACK_LOCK
     task = debug_task([
         (find_stale_packages, (path_lock, path_pack_lock), {'stale_months': 48}),
-        Interactive('poetry run pip list --outdated'),
+        Interactive('poetry run pip-check --cmd="poetry run pip" --hide-unchanged'),
     ])
     task['file_dep'].append(path_lock)
     task['targets'].append(path_pack_lock)
