@@ -27,20 +27,20 @@ def task_nox() -> DoitTask:
 
 @beartype
 def task_nox_test() -> DoitTask:
-    """Run all nox tests.
+    """Run the nox tests sessions.
 
     Returns:
         DoitTask: doit task
 
     """
     return debug_task([
-        Interactive('poetry run nox --session tests build_check'),
+        Interactive('poetry run nox --session tests'),
     ])
 
 
 @beartype
 def task_nox_coverage() -> DoitTask:
-    """Run all nox tests.
+    """Run the coverage session in nox.
 
     Returns:
         DoitTask: doit task
@@ -144,12 +144,15 @@ def task_coverage() -> DoitTask:
     """
     path_tests = DG.test.path_tests
     cov_dir = DG.test.path_coverage_index.parent
-    cov_html = f'--cov-report=html:"{cov_dir}"  --html="{DG.test.path_test_report}" --self-contained-html'
+    test_html = f'--html="{DG.test.path_test_report}" --self-contained-html'
     diff_html = f'--html-report {DG.test.path_diff_test_report}'
     return debug_task([
         Interactive(
-            f'poetry run python -m pytest "{path_tests}" {DG.test.args_pytest} --cov={DG.meta.pkg_name} {cov_html}',
+            f'poetry run coverage run --source={DG.meta.pkg_name} --module'
+            f' pytest "{path_tests}" {DG.test.args_pytest} {test_html}',
         ),
+        'poetry run python -m coverage report --show-missing',
+        f'poetry run python -m coverage html --directory={cov_dir}',
         'poetry run python -m coverage json',  # Create coverage.json file for "_write_coverage_to_md"
         'poetry run python -m coverage xml',
         Interactive(f'poetry run diff-cover coverage.xml {DG.test.args_diff} {diff_html}'),
@@ -185,13 +188,14 @@ def task_open_test_docs() -> DoitTask:
         DoitTask: doit task
 
     """
-    actions = [
-        (open_in_browser, (DG.test.path_coverage_index,)),
-        (open_in_browser, (DG.test.path_test_report,)),
+    paths = [
+        DG.test.path_test_report,
+        DG.test.path_diff_test_report,
+        DG.test.path_diff_lint_report,
+        DG.test.path_coverage_index,
+        DG.test.path_mypy_index,
     ]
-    if DG.test.path_mypy_index.is_file():
-        actions.append((open_in_browser, (DG.test.path_mypy_index,)))
-    return debug_task(actions)
+    return debug_task([(open_in_browser, (pth,)) for pth in paths if pth.is_file()])
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -210,35 +214,35 @@ def ptw_task(cli_args: str) -> DoitTask:
 
     """
     return {
-        'actions': [Interactive(f'poetry run ptw -- "{DG.test.path_tests}" {cli_args}')],
+        'actions': [Interactive(f'poetry run ptw "{DG.meta.path_project}" {cli_args}')],
         'verbosity': 2,
     }
 
 
 @beartype
-def task_ptw_not_chrome() -> DoitTask:
-    """Run pytest watch for failed first and skip the CHROME marker.
+def task_ptw_not_interactive() -> DoitTask:
+    """Run pytest watch for failed first and skip the INTERACTIVE marker.
 
-    kwargs: `-m 'not CHROME' -vvv`
+    kwargs: `-m 'not INTERACTIVE' -vvv`
 
     Returns:
         DoitTask: doit task
 
     """
-    return ptw_task('-m "not CHROME" -vvv')
+    return ptw_task('-m "not INTERACTIVE" -vvv')
 
 
 @beartype
 def task_ptw_ff() -> DoitTask:
-    """Run pytest watch for failed first and skip the CHROME marker.
+    """Run pytest watch for failed first and skip the INTERACTIVE marker.
 
-    kwargs: `--last-failed --new-first -m 'not CHROME' -vv`
+    kwargs: `--last-failed --new-first -m 'not INTERACTIVE' -vv`
 
     Returns:
         DoitTask: doit task
 
     """
-    return ptw_task('--last-failed --new-first -m "not CHROME" -vv')
+    return ptw_task('--last-failed --new-first -m "not INTERACTIVE" -vv')
 
 
 @beartype
